package hu.bme.mit.books.graphiti.editor.features;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.graphiti.features.IFeatureProvider;
import org.eclipse.graphiti.features.context.ICopyContext;
import org.eclipse.graphiti.mm.pictograms.Connection;
import org.eclipse.graphiti.mm.pictograms.PictogramElement;
import org.eclipse.graphiti.ui.features.AbstractCopyFeature;

import books.Author;
import books.Book;

public class CopyFeature extends AbstractCopyFeature {

	public CopyFeature(IFeatureProvider fp) {
		super(fp);
	}

	@Override
	public boolean canCopy(ICopyContext context) {
		final PictogramElement[] pes = context.getPictogramElements();
		if (pes == null || pes.length == 0) { // nothing selected
			return false;
		}

		// return true, if all selected elements are a EClasses
		for (PictogramElement pe : pes) {
			if (!(pe instanceof Connection)) {
				final Object bo = getBusinessObjectForPictogramElement(pe);
				if (!(bo instanceof Author || bo instanceof Book)) {
					return false;
				}
			}

		}
		return true;
	}

	@Override
	public void copy(ICopyContext context) {
		// get the business-objects for all pictogram-elements
		// we already verified, that all non-connection business-objects are
		// either Authors or Books
		PictogramElement[] pes = context.getPictogramElements();
		List<Object> bos = new ArrayList<Object>();
		for (int i = 0; i < pes.length; i++) {
			PictogramElement pe = pes[i];
			Object bo = getBusinessObjectForPictogramElement(pe);
			if (bo != null) {
				bos.add(bo);
			}
		}
		// put all business objects to the clipboard
		putToClipboard(bos.toArray(new Object[bos.size()]));
	}

}
