package hu.bme.mit.books.graphiti.editor.features;

import org.eclipse.graphiti.examples.common.ExampleUtil;
import org.eclipse.graphiti.features.IFeatureProvider;
import org.eclipse.graphiti.features.context.ICreateContext;
import org.eclipse.graphiti.features.impl.AbstractCreateFeature;
import org.eclipse.graphiti.mm.pictograms.Diagram;

import books.Book;
import books.BooksFactory;

public class CreateBookFeature extends AbstractCreateFeature {

	public CreateBookFeature(IFeatureProvider fp) {
		super(fp, "Book", "Creates a new book object");
	}

	@Override
	public boolean canCreate(ICreateContext context) {
		return context.getTargetContainer() instanceof Diagram;
	}

	@Override
	public Object[] create(ICreateContext context) {
        // ask user for EClass name
        String newBookName = ExampleUtil.askString("Create Book", "Enter Book Title", "");
        if (newBookName == null || newBookName.trim().length() == 0) {
            return EMPTY;
        }
 
        // create Book
        Book newBook = BooksFactory.eINSTANCE.createBook();
        // Add model element to resource.
        // We add the model element to the resource of the diagram for
        // simplicity's sake. Normally, a customer would use its own
        // model persistence layer for storing the business model separately.
        getDiagram().eResource().getContents().add(newBook);
        newBook.setTitle(newBookName);
 
        // Calling the add service
        addGraphicalRepresentation(context, newBook);
 
//        // activate direct editing after object creation
//        getFeatureProvider().getDirectEditingInfo().setActive(true);
        
        // return newly created business object(s)
        return new Object[] { newBook };
	}

}
